@@ -2,18 +2,44 @@ extern crate proc_macro;
 
 use proc_macro::TokenStream;
 use quote::quote;
-use syn;
+use syn::parse::{Parse, ParseStream};
+use syn::{parse, parse2, DeriveInput, Ident, Result};
 
-#[proc_macro_derive(Server)]
+struct ServerStruct {
+    name: Ident,
+}
+
+impl Parse for ServerStruct {
+    fn parse(input: ParseStream) -> Result<Self> {
+        Ok(ServerStruct {
+            name: input.parse()?,
+        })
+    }
+}
+
+#[proc_macro_derive(Server, attributes(struct_name))]
 pub fn checkout_server_derive(input: TokenStream) -> TokenStream {
-    let ast = syn::parse(input).unwrap();
+    let ast = parse(input).unwrap();
 
     // Build the trait implementation
     impl_checkout_server(&ast)
 }
 
-fn impl_checkout_server(ast: &syn::DeriveInput) -> TokenStream {
-    let name = &ast.ident;
+fn impl_checkout_server(ast: &DeriveInput) -> TokenStream {
+    let context_struct_name = &ast.ident;
+
+    let server_struct_attribute = ast
+        .attrs
+        .iter()
+        .filter(|a| a.path.segments.len() == 1 && a.path.segments[0].ident == "struct_name")
+        .nth(0)
+        .expect("struct_name attribute required to derive Server. This should be the name of the struct where ContextProvider is implemented");
+
+    let server_struct: ServerStruct =
+        parse2(server_struct_attribute.tokens.clone()).expect("invalid struct_name");
+
+    let server_struct_name = &server_struct.name;
+
     let gen = quote! {
         mod checkout_server {
             pub use checkout_core::dropshot::endpoint;
@@ -28,7 +54,7 @@ fn impl_checkout_server(ast: &syn::DeriveInput) -> TokenStream {
             pub use checkout_core::dropshot::TypedBody;
             pub use checkout_core::http::StatusCode;
             pub use checkout_core::{
-                error::Error, money::Currency, Checkout, Context, server::Server,
+                error::Error, money::Currency, Checkout, Context, ContextProvider, server::Server,
             };
 
             use std::sync::Arc;
@@ -36,7 +62,7 @@ fn impl_checkout_server(ast: &syn::DeriveInput) -> TokenStream {
             use checkout_core::server::CreateRequest;
 
             pub struct ServerContext {
-                pub context_provider: super::CheckoutServer
+                pub context_provider: super::#server_struct_name
             }
 
             #[endpoint {
@@ -74,7 +100,7 @@ fn impl_checkout_server(ast: &syn::DeriveInput) -> TokenStream {
             }
         }
 
-        impl checkout_server::Server<#name> for CheckoutServer {
+        impl checkout_server::Server<#context_struct_name> for #server_struct_name {
             type Context = checkout_server::ServerContext;
 
             fn api_description() -> checkout_server::ApiDescription<checkout_server::ServerContext> {
